//------------------------------------------------------- 2013-02-23 ----------
// DRO MINI  Version 1.0.2
//
// TMR0 -> LED column storobe
// TMR1 -> switch debounce (free run)
// TMR2 -> system-mode = NORMAL             : DSU signal disable
//         system-mode = OTHER_SETUP_DIRECT : LED sequential turn on interval
//
// MPU:        PIC18F2420
// Interrupts: TMR0,TMR2,RB4,RB5
// Frequency:  40MHz(10MHz x 4 PLL)
// MPLAB C18 
//                                          Written by HIROYUKI IIZUKA  JAPAN
//-----------------------------------------------------------------------------
#include <p18f2420.h>

#include <delays.h>
#include <stdio.h>
#include <stdlib.h>
#include <EEP.h>

void main (void);
void InterruptVectorLow (void);
void InterruptVectorHigh (void);
void Low_priority_interrupt (void);
void High_priority_interrupt (void);

#pragma config OSC     = HSPLL
#pragma config PWRT    = ON         // Power-up Timer
#pragma config BOREN   = ON
#pragma config BORV    = 3          // Brown-out Reset Voltage Minimum setting
#pragma config WDT     = OFF        // WDT disabled (control is placed on the SWDTEN bit)
#pragma config MCLRE   = OFF        // RE3 input pin enabled; MCLR pin disabled
#pragma config LVP     = OFF
#pragma config PBADEN  = OFF
#pragma config XINST   = OFF
#pragma config STVREN  = ON
#pragma config FCMEN   = OFF        // Fail-Safe Clock Monitor disabled
#pragma config DEBUG   = OFF
#pragma config IESO    = OFF
#pragma config CP0     = OFF
#pragma config CP1     = OFF
#pragma config CPB     = OFF
#pragma config CPD     = OFF
#pragma config WRT0    = OFF
#pragma config WRT1    = OFF
#pragma config WRTC    = OFF
#pragma config WRTB    = OFF
#pragma config WRTD    = OFF
#pragma config EBTR0   = OFF
#pragma config EBTR1   = OFF
#pragma config EBTRB   = OFF

//---- (7segment LED) device definitions -------------
#define LED_DEV1_TRIS    TRISBbits.RB2
#define LED_DEV2_TRIS    TRISBbits.RB3
#define LED_DEV3_TRIS    TRISBbits.RB4
#define LED_DEV4_TRIS    TRISBbits.RB5
#define LED_DEV5_TRIS    TRISBbits.RB6
#define LED_DEV6_TRIS    TRISBbits.RB7

#define LED_DEV1         LATBbits.LATB2
#define LED_DEV2         LATBbits.LATB3
#define LED_DEV3         LATBbits.LATB4
#define LED_DEV4         LATBbits.LATB5
#define LED_DEV5         LATBbits.LATB6
#define LED_DEV6         LATBbits.LATB7

#define LED_SEG_A_TRIS   TRISAbits.RA3
#define LED_SEG_B_TRIS   TRISAbits.RA4
#define LED_SEG_C_TRIS   TRISAbits.RA5
#define LED_SEG_D_TRIS   TRISCbits.RC0
#define LED_SEG_E_TRIS   TRISCbits.RC1
#define LED_SEG_F_TRIS   TRISCbits.RC2
#define LED_SEG_G_TRIS   TRISCbits.RC6
#define LED_SEG_DP_TRIS  TRISCbits.RC7

#define LED_SEG_A        LATAbits.LATA3
#define LED_SEG_B        LATAbits.LATA4
#define LED_SEG_C        LATAbits.LATA5
#define LED_SEG_D        LATCbits.LATC0
#define LED_SEG_E        LATCbits.LATC1
#define LED_SEG_F        LATCbits.LATC2
#define LED_SEG_G        LATCbits.LATC6
#define LED_SEG_DP       LATCbits.LATC7
 
//---- (Digital Caliper) device definitions -------
#define DSU_CLOCK_TRIS   TRISBbits.RB1
#define DSU_CLOCK        PORTBbits.RB1
#define DSU_CLOCK_PORT   PORTB
#define DSU_CLOCK_BIT    1

#define DSU_DATA_TRIS    TRISBbits.RB0
#define DSU_DATA         PORTBbits.RB0
#define DSU_DATA_PORT    PORTB
#define DSU_DATA_BIT     0

#define DSU_ZERO_TRIS    TRISAbits.RA0   // CLOCK RESET
#define DSU_ZERO         LATAbits.LATA0

#define DSU_MODE_TRIS    TRISAbits.RA1   // DATA RESET
#define DSU_MODE         LATAbits.LATA1

#define dsu_colck_changed          INTCON3bits.INT1IF
#define dsu_colck_int_enable       INTCON3bits.INT1IE
#define dsu_colck_int_rising_edge  INTCON2bits.INTEDG1

#define dsu_data_changed          INTCONbits.INT0IF
#define dsu_data_int_enable       INTCONbits.INT0IE
#define dsu_data_int_rising_edge  INTCON2bits.INTEDG0

//---- (Push Switch) device definitions ----------
// #define SW1_TRIS         TRISEbits.RE3 // MCLRE only 40pin device
#define SW1              PORTEbits.RE3

#define SW2_TRIS         TRISAbits.RA2
#define SW2              PORTAbits.RA2

//-------- TMR0 --------------------------------------------------------------
#define Timer0_overflowd              INTCONbits.TMR0IF
#define Timer0_interrupt_enable       INTCONbits.TMR0IE
#define Timer0_priority               INTCON2bits.TMR0IP
#define Timer0_enable                 T0CONbits.TMR0ON
#define Timer0_mode_8bit              T0CONbits.T08BIT
#define Timer0_use_external_clock     T0CONbits.T0CS
#define Timer0_prescale_NOT_assign    T0CONbits.PSA
#define Timer0_prescale_bit2          T0CONbits.T0PS2
#define Timer0_prescale_bit1          T0CONbits.T0PS1
#define Timer0_prescale_bit0          T0CONbits.T0PS0

//-------- TMR1 --------------------------------------------------------------
#define Timer1_overflowd              PIR1bits.TMR1IF 
#define Timer1_interrupt_enable       PIE1bits.TMR1IE 
#define Timer1_prescale_bit0          T1CONbits.T1CKPS1
#define Timer1_prescale_bit1          T1CONbits.T1CKPS0
#define Timer1_use_external_clock     T1CONbits.TMR1CS
#define Timer1_enable                 T1CONbits.TMR1ON
#define Timer1_priority               IPR1bits.TMR1IP

//-------- TMR2 --------------------------------------------------------------
#define Timer2_overflowd              PIR1bits.TMR2IF 
#define Timer2_interrupt_enable       PIE1bits.TMR2IE
#define Timer2_priority               IPR1bits.TMR2IP
#define Timer2_postscale_bit3         T2CONbits.T2OUTPS3
#define Timer2_postscale_bit2         T2CONbits.T2OUTPS2
#define Timer2_postscale_bit1         T2CONbits.T2OUTPS1
#define Timer2_postscale_bit0         T2CONbits.T2OUTPS0
#define Timer2_enable                 T2CONbits.TMR2ON
#define Timer2_prescale_bit1          T2CONbits.T2CKPS1
#define Timer2_prescale_bit0          T2CONbits.T2CKPS0
#define Timer2_prescale               T2CONbits.T2CKPS

//---- symbol definisions ----------------------------
#define High_priority   1  // interrupt
#define Low_priority    0

#define PORT_OUTPUT     0
#define PORT_INPUT      1

#define LED_ON          0  // 7segment LED anode common 
#define LED_OFF         1

#define HIGH            1
#define LOW             0

#define MAG_2X          0x00
#define MAG_1X          0xFF

#define CHANGED         1
#define NOT_CHANGED     0

#define YES             1
#define NO              0

#define PLUS            0x00
#define MINUS           0xFF

#define PUSHED          0x00
#define RELEASED        0x01

#define KEY_HALF        1  // Switch No
#define KEY_ZERO        2

#define SW_ON           0
#define SW_OFF          1

//---- EEPROM ADDRESS MAP ----------------------------
#define EEADR_INIT       0x00   // $00:calibrated   $FF:not calibrated
#define EEADR_DIRECT     0x01
#define EEADR_DOUBLE     0x02   // MAG_2X / MAG_1X
#define EEADR_DSU_TYPE   0x03   // 1:24bit 2:BCD 3:20bit

//---- SCALE TYPE ------------------------------------
#define  BIT24         1     // 24bit binary
#define  BCD           2     // BCD
#define  BIT20         3     // 20bit binary  + 4bit flag
#define  DISCON        4     // disconect scale

//---- debounce wait ---------------------------------
#define DEBOUNCE_WAIT  20	 // 20ms

//---- 7segment LED character definitions ----------------
// segment-A  -> bit0
//   :
// segment-G  -> bit6
// segment-DP -> bit7

#define Digit_0      0xC0
#define Digit_1      0xF9
#define Digit_2      0xA4
#define Digit_3      0xB0
#define Digit_4      0x99
#define Digit_5      0x92
#define Digit_6      0x82
#define Digit_7      0xF8
#define Digit_8      0x80
#define Digit_9      0x90
#define Digit_E      0x86
#define Digit_F      0x8E
#define Digit_U      0xC1
#define Digit_u      0xE3
#define Digit_L      0xC7
#define Digit_C      0xC6
#define Digit_P      0x8C
#define Digit_Y      0x91
#define Digit_A      0x88
#define Digit_r      0xAF
#define Digit_n      0xAB
#define Digit_d      0xA1
#define Digit_t      0x87
#define Digit_b      0x83
#define Digit_i      0xEF
#define Digit_l      0xF9
#define Digit_NON    0xFF
#define Digit_o      0xA3
#define Digit_S      0x92
#define Digit_h      0x8B
#define Digit_c      0xA7
#define Digit_US     0xF7
#define Digit_MINUS  0xBF
#define Digit_DP     0x7F

//-------  typedefs ------------
typedef union {
  unsigned long  Lval;
  long           Lvalsign;
  unsigned short Wval[2];
  unsigned char  Bval[4];
} access_struc_long; 

typedef union {
  unsigned short Wval;
  unsigned char  Bval[2];
} access_struc_wd; 

typedef  unsigned short byte;
typedef  unsigned char  boolean;
#define true  1
#define false 0

//---- system mode -----------------------------------
enum type_system_mode {
  NORMAL, 
  OTHER_SETUP_DIRECT, 
  OTHER_SETUP_DOUBLE, 
  OTHER_SETUP_DSU_TYPE
};

struct {
  unsigned POWER_ON_BIT:1;
  unsigned DSU_SIGNAL_BIT:1;
  unsigned POWER_ON_BIT2:1;   // dsu_type = BCD
  unsigned :5;
} Dsu_stat;


#define stop()  while(1){;}

//---- DSU Global variable definitions for assembler routine ----
access_struc_long Dsudat;
access_struc_long Wk;

byte Dsu_bit_pos;
byte Dsu_data_tmp;

byte Dummy;

byte Led_pos;
byte Tmr2_ovfc;   //  TMR2 overflow counter
byte Char_pos;
byte Sys_mode;    //  enum  Normal/other_setup_direct/other_setup_double/other_setup_dsu_type
byte Dsu_direct;  //  0:<--inc--  1:--inc-->
byte Dsu_dbl;     //  MAG_2X / MAG_1X
byte Dsu_type;    //  24bit / BCD / 20bit / discon

byte Led_val1;
byte Led_val2;
byte Led_val3;
byte Led_val4;
byte Led_val5;
byte Led_val6;

unsigned long Big_raw;
unsigned long Small_raw;
byte Big_raw_count;
byte Small_raw_count;

unsigned long Prev1_raw;
unsigned long Prev2_raw;
unsigned long Div_raw;
byte Stop_count;

byte Tmr2_overflow_cnt;


//-------------------------------------------------------------------------------
// delay  msec           Clock = 48Mhz
//-------------------------------------------------------------------------------
void delay_ms(unsigned short delay)
{
  unsigned short i;

  for( i = 0 ; i <= delay ; i++ ) {
    Delay1KTCYx(12);
  }
}

//----------------------------------------------------------------------------
// 7seg decode -> segment port
//----------------------------------------------------------------------------
void decode_7seg( byte code )
{
  LED_SEG_A  = code & 0x01;  code >>= 1;
  LED_SEG_B  = code & 0x01;  code >>= 1;
  LED_SEG_C  = code & 0x01;  code >>= 1;
  LED_SEG_D  = code & 0x01;  code >>= 1;
  LED_SEG_E  = code & 0x01;  code >>= 1;
  LED_SEG_F  = code & 0x01;  code >>= 1;
  LED_SEG_G  = code & 0x01;  code >>= 1;
  LED_SEG_DP = code & 0x01;
}


//----------------------------------------------------------------------------
// set LOW & HIGH interrupt vector
//----------------------------------------------------------------------------
#pragma code low_vector = 0x18
void InterruptVectorLow (void)
{
  _asm
    goto Low_priority_interrupt   //jump to interrupt routine
  _endasm
}
#pragma code

#pragma code InterruptVectorHigh = 0x08
void InterruptVectorHigh (void)
{
  _asm
    goto High_priority_interrupt  //jump to interrupt routine
  _endasm
}
#pragma code

//----------------------------------------------------------------------------
// Low priority interrupt routine
//----------------------------------------------------------------------------
#pragma code
#pragma interruptlow Low_priority_interrupt

void Low_priority_interrupt ()
{
  if( Timer0_overflowd == 1 ) {
    switch( Led_pos ) {
      case 1: LED_DEV6 = LED_OFF; decode_7seg( Led_val1 ); LED_DEV1 = LED_ON;  Led_pos++; break;
      case 2: LED_DEV1 = LED_OFF; decode_7seg( Led_val2 ); LED_DEV2 = LED_ON;  Led_pos++; break;
      case 3: LED_DEV2 = LED_OFF; decode_7seg( Led_val3 ); LED_DEV3 = LED_ON;  Led_pos++; break;
      case 4: LED_DEV3 = LED_OFF; decode_7seg( Led_val4 ); LED_DEV4 = LED_ON;  Led_pos++; break;
      case 5: LED_DEV4 = LED_OFF; decode_7seg( Led_val5 ); LED_DEV5 = LED_ON;  Led_pos++; break;
      case 6: LED_DEV5 = LED_OFF; decode_7seg( Led_val6 ); LED_DEV6 = LED_ON;  Led_pos=1; break;
    }

    TMR0L = 0;
    Timer0_overflowd = 0;   // Clear interrupt flag
  }

  if( Timer2_overflowd == 1 ) {
    Timer2_overflowd = 0;  // Clear TMR2 interrupt flag
    if( Sys_mode == OTHER_SETUP_DIRECT ) {
      TMR2 = 0;
      if( Tmr2_ovfc == 8 ) {
        Tmr2_ovfc = 0;
      } else {
        Tmr2_ovfc++;
      }
      if( Tmr2_ovfc == 7 ) {
        if( Dsu_direct == PLUS ) {
          Char_pos++;
          if( Char_pos == 7 ) {
            Char_pos = 1;
          }
        } else {
          Char_pos--;
          if( Char_pos == 0 ) {
            Char_pos = 6;
          }
        }
        Led_val1 = Digit_NON;
        Led_val2 = Digit_NON;
        Led_val3 = Digit_NON;
        Led_val4 = Digit_NON;
        Led_val5 = Digit_NON;
        Led_val6 = Digit_NON;
        switch( Char_pos ) {
          case 1: Led_val1 = Digit_8; break;
          case 2: Led_val2 = Digit_8; break;
          case 3: Led_val3 = Digit_8; break;
          case 4: Led_val4 = Digit_8; break;
          case 5: Led_val5 = Digit_8; break;
          case 6: Led_val6 = Digit_8; break;
        }
      }
    }

    if( Sys_mode == NORMAL ) {
      switch( Dsu_type ) {
        case BIT24:
          Timer2_enable = 0;  // STOP  TMR2
          Dummy = PORTB;
          dsu_colck_changed    = 0;  // Clear interrupt flag
          dsu_colck_int_enable = 1;  // Enable interrupt
          break;
        case BCD:
          Timer2_enable = 0;  // STOP  TMR2
          Dummy = PORTB;
          dsu_colck_changed    = 0;  // Clear interrupt flag
          dsu_colck_int_enable = 1;  // Enable interrupt
          break;
        case BIT20:
          break;
      }  
    }
  }
}

//----------------------------------------------------------------------------
// High priority interrupt routine
//----------------------------------------------------------------------------
#pragma code
#pragma interrupt High_priority_interrupt

#pragma udata BANK1=0x100
byte _dsu_count;
byte _dsu_data1;
byte _dsu_data2;
byte _dsu_data3;
byte _dsu_data4;
byte bsr_save;
#pragma udata

void High_priority_interrupt ()
{
  if( dsu_colck_changed == 1 ) {
    switch( Dsu_type ) {
      case BIT24:
        dsu_colck_int_enable = 0;         // Disable INT1 interrupt
        if( Sys_mode == NORMAL ) {
              _asm
              MOVFF  BSR,bsr_save
              MOVLB  1    // bank1
         //------- receive 1st byte ----------------------
              movlw  8
              movwf  _dsu_count,BANKED
              // Detect Clock Low -> High
CLOCK_LOOP_L:
              rrcf    _dsu_data1,1,BANKED
LTOH_L:
              btfss  DSU_CLOCK_PORT,DSU_CLOCK_BIT,ACCESS        // Low -> High then NEXT inst
              goto   LTOH_L
             // set DATA
              btfsc  DSU_DATA_PORT,DSU_DATA_BIT,ACCESS
              goto   SET_HIGH_L
              goto   SET_LOW_L
SET_HIGH_L:
              bsf    _dsu_data1,7,BANKED
              goto   HTOL_L
SET_LOW_L:
              bcf    _dsu_data1,7,BANKED
             // Detect Clock High -> Low
HTOL_L:
              btfsc  DSU_CLOCK_PORT,DSU_CLOCK_BIT,ACCESS       // High -> Low then NEXT inst
              goto   HTOL_L
              decfsz _dsu_count,1,BANKED
              goto   CLOCK_LOOP_L

          //------- receive 2nd byte ----------------------
              movlw  8
              movwf  _dsu_count,BANKED
             // Detect Clock Low -> High
CLOCK_LOOP_M:
              rrcf    _dsu_data2,1,BANKED
LTOH_M:
              btfss  DSU_CLOCK_PORT,DSU_CLOCK_BIT,ACCESS      // Low -> High then NEXT inst
              goto   LTOH_M
             // set DATA
              btfsc  DSU_DATA_PORT,DSU_DATA_BIT,ACCESS
              goto   SET_HIGH_M
              goto   SET_LOW_M
SET_HIGH_M:
              bsf    _dsu_data2,7,BANKED
              goto   HTOL_M
SET_LOW_M:
              bcf    _dsu_data2,7,BANKED
             // Detect Clock High -> Low
HTOL_M:
              btfsc  DSU_CLOCK_PORT,DSU_CLOCK_BIT,ACCESS        // High -> Low then NEXT inst
              goto   HTOL_M
              decfsz _dsu_count,1,BANKED
              goto   CLOCK_LOOP_M
          //------- receive 3rd byte ----------------------
              movlw  8
              movwf  _dsu_count,BANKED
             // Detect Clock Low -> High
CLOCK_LOOP_H:
              rrcf    _dsu_data3,1,BANKED
LTOH_H:
              btfss  DSU_CLOCK_PORT,DSU_CLOCK_BIT,ACCESS        // Low -> High then NEXT inst
              goto   LTOH_H
             // set DATA
              btfsc  DSU_DATA_PORT,DSU_DATA_BIT,ACCESS
              goto   SET_HIGH_H
              goto   SET_LOW_H
SET_HIGH_H:
              bsf    _dsu_data3,7,BANKED
              goto   HTOL_H
SET_LOW_H:
              bcf    _dsu_data3,7,BANKED
             // Detect Clock High -> Low
HTOL_H:
              btfsc  DSU_CLOCK_PORT,DSU_CLOCK_BIT,ACCESS       // High -> Low then NEXT inst
              goto   HTOL_H
              decfsz _dsu_count,1,BANKED
              goto   CLOCK_LOOP_H
        //-----------------------------------------------
              rrcf    _dsu_data3,1,BANKED
              rrcf    _dsu_data2,1,BANKED
              rrcf    _dsu_data1,1,BANKED
              rrcf    _dsu_data3,1,BANKED
              rrcf    _dsu_data2,1,BANKED
              rrcf    _dsu_data1,1,BANKED
              rrcf    _dsu_data3,1,BANKED
              rrcf    _dsu_data2,1,BANKED
              rrcf    _dsu_data1,1,BANKED
              movff  bsr_save,BSR
              _endasm

          if( (_dsu_data3 & 0b00010000) == 0b00010000 ) {
            _dsu_data3 |= 0b11100000;
          } else {
            _dsu_data3 &= 0b00011111;
          }

          Dsudat.Bval[0] = _dsu_data1;
          Dsudat.Bval[1] = _dsu_data2;
          Dsudat.Bval[2] = _dsu_data3;
          Dsudat.Bval[3] = _dsu_data4;

          Dsu_stat.DSU_SIGNAL_BIT = CHANGED;

          TMR2 = 139;               // 139 = 3 ms
          Timer2_overflowd = 0;     // Clear TMR2 interrupt flag
          Timer2_enable = 1;        // START TMR2
        }
        break;

      case BCD:
        dsu_colck_int_enable = 0;         // Disable INT1 interrupt
        if( Sys_mode == NORMAL ) {
              _asm
              MOVFF  BSR,bsr_save
              MOVLB  1    // bank1
          //------- receive 1st byte ----------------------
              movlw  8
              movwf  _dsu_count,BANKED
              // Detect Clock Low -> High
CLOCK_LOOP_L_BCD:
              rrcf    _dsu_data1,1,BANKED
LTOH_L_BCD:
              btfss  DSU_CLOCK_PORT,DSU_CLOCK_BIT,ACCESS                // Low -> High then NEXT inst
              goto   LTOH_L_BCD
             // set DATA
              btfsc  DSU_DATA_PORT,DSU_DATA_BIT,ACCESS
              goto   SET_HIGH_L_BCD
              goto   SET_LOW_L_BCD
SET_HIGH_L_BCD:
              bsf    _dsu_data1,7,BANKED
              goto   HTOL_L_BCD
SET_LOW_L_BCD:
              bcf    _dsu_data1,7,BANKED
             // Detect Clock High -> Low
HTOL_L_BCD:
              btfsc  DSU_CLOCK_PORT,DSU_CLOCK_BIT,ACCESS                // High -> Low then NEXT inst
              goto   HTOL_L_BCD
              decfsz _dsu_count,1,BANKED
              goto   CLOCK_LOOP_L_BCD
          //------- receive 2nd byte ----------------------
              movlw  8
              movwf  _dsu_count,BANKED
             // Detect Clock Low -> High
CLOCK_LOOP_M_BCD:
              rrcf    _dsu_data2,1,BANKED
LTOH_M_BCD:
              btfss  DSU_CLOCK_PORT,DSU_CLOCK_BIT,ACCESS               // Low -> High then NEXT inst
              goto   LTOH_M_BCD
             // set DATA
              btfsc  DSU_DATA_PORT,DSU_DATA_BIT,ACCESS
              goto   SET_HIGH_M_BCD
              goto   SET_LOW_M_BCD
SET_HIGH_M_BCD:
              bsf    _dsu_data2,7,BANKED
              goto   HTOL_M_BCD
SET_LOW_M_BCD:
              bcf    _dsu_data2,7,BANKED
             // Detect Clock High -> Low
HTOL_M_BCD:
              btfsc  DSU_CLOCK_PORT,DSU_CLOCK_BIT,ACCESS              // High -> Low then NEXT inst
              goto   HTOL_M_BCD
              decfsz _dsu_count,1,BANKED
              goto   CLOCK_LOOP_M_BCD
          //------- receive 3rd byte ----------------------
              movlw  8
              movwf  _dsu_count,BANKED
          // Detect Clock Low -> High
CLOCK_LOOP_H_BCD:
              rrcf    _dsu_data3,1,BANKED
LTOH_H_BCD:
              btfss  DSU_CLOCK_PORT,DSU_CLOCK_BIT,ACCESS             // Low -> High then NEXT inst
              goto   LTOH_H_BCD
          // set DATA
              btfsc  DSU_DATA_PORT,DSU_DATA_BIT,ACCESS
              goto   SET_HIGH_H_BCD
              goto   SET_LOW_H_BCD
SET_HIGH_H_BCD:
              bsf    _dsu_data3,7,BANKED
              goto   HTOL_H_BCD
SET_LOW_H_BCD:
              bcf    _dsu_data3,7,BANKED
          // Detect Clock High -> Low
HTOL_H_BCD:
              btfsc  DSU_CLOCK_PORT,DSU_CLOCK_BIT,ACCESS            // High -> Low then NEXT inst
              goto   HTOL_H_BCD
              decfsz _dsu_count,1,BANKED
              goto   CLOCK_LOOP_H_BCD
        //------------------------------------------------
              movlw  4
              movwf  _dsu_count,BANKED
          // Detect Clock Low -> High
CLOCK_LOOP_HH_BCD:
              rrcf    _dsu_data4,1,BANKED
LTOH_HH_BCD:
              btfss  DSU_CLOCK_PORT,DSU_CLOCK_BIT,ACCESS            // Low -> High then NEXT inst
              goto   LTOH_HH_BCD
          // set DATA
              btfsc  DSU_DATA_PORT,DSU_DATA_BIT,ACCESS
              goto   SET_HIGH_HH_BCD
              goto   SET_LOW_HH_BCD
SET_HIGH_HH_BCD:
              bsf    _dsu_data4,7,BANKED
              goto   HTOL_HH_BCD
SET_LOW_HH_BCD:
              bcf    _dsu_data4,7,BANKED
          // Detect Clock High -> Low
HTOL_HH_BCD:
              btfsc  DSU_CLOCK_PORT,DSU_CLOCK_BIT,ACCESS           // High -> Low then NEXT inst
              goto   HTOL_HH_BCD
              decfsz _dsu_count,1,BANKED
              goto   CLOCK_LOOP_HH_BCD
        //------------------------------------------------
              movff  bsr_save,BSR
              _endasm

          Dsudat.Bval[0] = _dsu_data1;
          Dsudat.Bval[1] = _dsu_data2;
          Dsudat.Bval[2] = _dsu_data3;
          Dsudat.Bval[3] = _dsu_data4;

          Dsu_stat.DSU_SIGNAL_BIT = CHANGED;

          TMR2 = 139;               // 139 = 3 ms
          Timer2_overflowd = 0;     // Clear TMR2 interrupt flag
          Timer2_enable = 1;        // START TMR2
        }
        break;

      case BIT20:
        if( Sys_mode == NORMAL ) {
          Dsu_data_tmp >>= 1;
          if( DSU_DATA == 0 ) {
            Dsu_data_tmp |= 0b10000000;
          } else {
            Dsu_data_tmp &= 0b01111111;
          }

          switch( Dsu_bit_pos ) {
            case 7:
              _dsu_data1 = Dsu_data_tmp;
              break;
            case 15:
              _dsu_data2 = Dsu_data_tmp;
              break;
            case 23:
              _dsu_data3 = Dsu_data_tmp;
              Dsudat.Bval[0] = _dsu_data1;
              Dsudat.Bval[1] = _dsu_data2;
              Dsudat.Bval[2] = _dsu_data3;
              Dsudat.Bval[3] = 0x00;
              Dsu_stat.DSU_SIGNAL_BIT = CHANGED;
              break;
          } 
          Dsu_bit_pos++;
          dsu_colck_changed = 0;  // Clear INT1(PORTB1) interrupt flag
        }
        break;
    }
  }

  if( dsu_data_changed == 1 ) {
    if( DSU_CLOCK == LOW ) {
      Dsu_bit_pos = 0;  // detect end of data
    }
    dsu_data_changed = 0;   // Clear INT0(PORTB0) interrupt flag
  }
}

//===============================================================================
// decode ascii-char -> 7segment-LED
//
// ArgNo  name    type    contents
// ------ ------- ------- ------------------------
// arg1   ascii   byte    ascii charactor
// RETURN         byte    7-SEGMENT LED segment pattern
//===============================================================================
byte led7seg_decode( byte ascii )
{
  byte result;

  switch( ascii ) {
    case '-':  result = Digit_MINUS; break;
    case ' ':  result = Digit_NON;   break;
    case '0':  result = Digit_0;     break;
    case '1':  result = Digit_1;     break;
    case '2':  result = Digit_2;     break;
    case '3':  result = Digit_3;     break;
    case '4':  result = Digit_4;     break;
    case '5':  result = Digit_5;     break;
    case '6':  result = Digit_6;     break;
    case '7':  result = Digit_7;     break;
    case '8':  result = Digit_8;     break;
    case '9':  result = Digit_9;     break;
    default:   result = Digit_E;     break;
  }

  return result;
}

//===============================================================================
// TMR1 set
//
// ArgNo  name    type    contents
// ------ ------- ------- ------------------------
// arg1   msec    word    mili seconds
//===============================================================================
void tmr1_set( unsigned short msec )
{
  access_struc_wd tmr_val;

  tmr_val.Wval = 0xFFFF - (1250 * msec);
  Timer1_overflowd = 0;
  TMR1H = tmr_val.Bval[0];
  TMR1L = tmr_val.Bval[0];
}

//===============================================================================
// SW1-SW3 debounce routine
//
// ArgNo  name     type    contents
// ------ -------- ------- -----------------------------------
// arg1   sw_no    byte    Switch No 1 to 3
// RETURN -        byte    1:PUSHED 0:REREASED
//===============================================================================
byte keyin_through( byte sw_no)
{
  byte sw_stat;

  switch( sw_no ) {
    case 1: sw_stat = SW1; break;
    case 2: sw_stat = SW2; break;
  }

  tmr1_set( DEBOUNCE_WAIT );
  while( 1 ) {
    switch( sw_no ) {
      case 1:
        if( SW1 != sw_stat ) {
          if( sw_stat == LOW ) {
            sw_stat = HIGH;
          } else {
            sw_stat = LOW;
          }
          tmr1_set( DEBOUNCE_WAIT );
        }
        break;
      case 2:
        if( SW2 != sw_stat ) {
          if( sw_stat == LOW ) {
            sw_stat = HIGH;
          } else {
            sw_stat = LOW;
          }
          tmr1_set( DEBOUNCE_WAIT );
        }
        break;
    } 

    if( Timer1_overflowd == 1 ) {
      break;
    } 
  }

  return sw_stat;
}

//===============================================================================
// convert BCD -> byte
//
// ArgNo  name     type    contents
// ------ -------- ------- -----------------------------------
// arg1            byte    BCD
// RETURN          byte    bin
//===============================================================================
byte Bcd2Dec( byte bcd )
{
  byte tmp;

  tmp = bcd;
  tmp >>= 4;
  tmp &= 0x0F;
  tmp *= 10;

  return tmp + (bcd & 0x0F );
}

//===============================================================================
// get dsu data
//
// ArgNo  name     type    contents
// ------ -------- ------- -----------------------------------
// RETURN          longint dsu 32 bit data
//===============================================================================
long get_dsu_rawdata(void)
{
  byte tmp_byte1;          // BCD
  byte tmp_byte2;          // BCD
  byte tmp_byte3;          // BCD
  long tmp_longint1;       // BCD
  long tmp_longint2;       // BCD
  long tmp_longint3;       // BCD
  long tmp_longint4;       // BCD
  access_struc_long wk;
  long result;

  switch( Dsu_type ) {
    case BIT24:
      wk.Lval = Dsudat.Lval;
      if( (wk.Bval[2] & 0b10000000 ) == 0b10000000 ) {
        wk.Bval[3] = 0xFF;
      } else {
        wk.Bval[3] = 0x00;
      }
      result = wk.Lval;
      break;

    case BCD:
      tmp_longint1 = 0;
      tmp_longint2 = 0;
      tmp_longint3 = 0;
      tmp_longint4 = 0;

      tmp_byte1 = Bcd2Dec(Dsudat.Bval[0] ^ 0xFF);
      tmp_byte2 = Bcd2Dec(Dsudat.Bval[1] ^ 0xFF);
      tmp_byte3 = Bcd2Dec(Dsudat.Bval[2] ^ 0xFF);

      tmp_longint1 = (long)tmp_byte1;
      tmp_longint2 = (long)tmp_byte2 * 100;
      tmp_longint3 = (long)tmp_byte3 * 10000;

      tmp_longint4 = tmp_longint3 + tmp_longint2 + tmp_longint1;

      if( (Dsudat.Bval[3] & 0b00010000 ) == 0b00010000 ) {
        tmp_longint4 = -tmp_longint4;
      }
      result = tmp_longint4;
      break;

    case BIT20:
      wk.Lval = Dsudat.Lval;
      wk.Bval[2] &= 0x0F;
      wk.Bval[3] =  0x00;

      if( (Dsudat.Bval[2] & 0b0010000 ) == 0b0010000 ) {
        result = -wk.Lvalsign;
      } else {
        result = wk.Lvalsign;
      }
      break;
  }

  return result;
}

//===============================================================================
// display numeric 6 digits LED
//
// ArgNo  name     type    contents
// ------ -------- ------- -----------------------------------
// arg1   disp_val longint display 32bit value
//===============================================================================
void disp_led( long disp_val )
{
  char led_txt[8];
  unsigned char sign;

  if( disp_val < 0 ) {
    sign = MINUS;
    disp_val = -disp_val;
  } else {
    sign = PLUS;
  }

  if( Dsu_dbl == MAG_2X ) {
    disp_val <<= 1;
  }

  sprintf( led_txt, (rom char far const *)"%6ld", disp_val );
  Led_val1 = led7seg_decode( led_txt[5] );
  Led_val2 = led7seg_decode( led_txt[4] );
  Led_val3 = led7seg_decode( led_txt[3] );
  Led_val4 = led7seg_decode( led_txt[2] );
  Led_val5 = led7seg_decode( led_txt[1] );
  Led_val6 = led7seg_decode( led_txt[0] );

  Led_val3 &= Digit_DP;
  if( sign == MINUS ) {
    Led_val1 &= Digit_DP;
    if( Led_val6 == Digit_NON ) {
       Led_val6 = Digit_MINUS;
    }
  }
   
  if( Dsu_type == BCD ) {
    if( (Dsudat.Bval[3] & 0b01000000 ) == 0b01000000 ) {
      Led_val6 = Digit_NON;
      Led_val5 = Digit_i;
      Led_val4 = Digit_n;
      Led_val3 = Digit_c;
      Led_val2 = Digit_h;
      Led_val1 = Digit_NON;
    }
  }
}

//===============================================================================
// Display "SAVE"
//===============================================================================
void disp_save(void)
{
  Led_val6 = Digit_NON;
  Led_val5 = Digit_S;
  Led_val4 = Digit_A;
  Led_val3 = Digit_U;
  Led_val2 = Digit_E;
  Led_val1 = Digit_NON;
}

//===============================================================================
// Other setup
//===============================================================================
void other_setup(void)
{
  Led_val6 = Digit_S;
  Led_val5 = Digit_E;
  Led_val4 = Digit_t;
  Led_val3 = Digit_NON;
  Led_val2 = Digit_NON;
  Led_val1 = Digit_NON;
  delay_ms(1000);

  //--------------------------------------
  // direct
  //--------------------------------------
  Timer2_enable = 1;  // START TMR2
  Sys_mode = OTHER_SETUP_DIRECT;
  while( keyin_through(KEY_ZERO) == SW_ON ) { ; }
  while( keyin_through(KEY_HALF) == SW_OFF ) {
    if( keyin_through(KEY_ZERO) == SW_ON ) {
      if( Dsu_direct == PLUS ) {
        Dsu_direct = MINUS;
      } else {
        Dsu_direct = PLUS;
      }
      while( keyin_through(KEY_ZERO) == SW_ON ) { ; }
    }
  }

  if( Dsu_direct == PLUS ) {
    Write_b_eep( EEADR_DIRECT, PLUS  );
  } else {
    Write_b_eep( EEADR_DIRECT, MINUS );
  }
  delay_ms(20);

  Timer2_enable = 0;     // STOP  TMR2
  Timer2_overflowd = 0;  // Clear TMR2 interrupt flag
  disp_save();
  while( keyin_through(KEY_HALF) == SW_ON ) { ; }
    
  //--------------------------------------
  // x 2
  //--------------------------------------
  Sys_mode = OTHER_SETUP_DOUBLE;
  delay_ms(1000);

  if( Dsu_dbl == MAG_2X ) {
    Led_val6 = Digit_d;
    Led_val5 = Digit_o;
    Led_val4 = Digit_u;
    Led_val3 = Digit_b;
    Led_val2 = Digit_L;
    Led_val1 = Digit_E;
  } else {
    Led_val6 = Digit_MINUS;
    Led_val5 = Digit_MINUS;
    Led_val4 = Digit_MINUS;
    Led_val3 = Digit_MINUS;
    Led_val2 = Digit_MINUS;
    Led_val1 = Digit_MINUS;
  }
  while( keyin_through(KEY_HALF) == SW_OFF ) {
    if( keyin_through(KEY_ZERO) == SW_ON ) {
      if( Dsu_dbl == MAG_1X ) {
        Dsu_dbl = MAG_2X;
        Led_val6 = Digit_d;
        Led_val5 = Digit_o;
        Led_val4 = Digit_u;
        Led_val3 = Digit_b;
        Led_val2 = Digit_L;
        Led_val1 = Digit_E;
      } else {
        Dsu_dbl = MAG_1X;
        Led_val6 = Digit_MINUS;
        Led_val5 = Digit_MINUS;
        Led_val4 = Digit_MINUS;
        Led_val3 = Digit_MINUS;
        Led_val2 = Digit_MINUS;
        Led_val1 = Digit_MINUS;
      }
      while( keyin_through(KEY_ZERO) == SW_ON ) { ; }
    }
  }

  if( Dsu_dbl == MAG_1X ) {
    Write_b_eep( EEADR_DOUBLE, MAG_1X );
  } else {
    Write_b_eep( EEADR_DOUBLE, MAG_2X );
  }
  delay_ms(20);
  disp_save();
  delay_ms(1000);

  //--------------------------------------
  // dsu type        BIT24 / BCD / BIT20
  //--------------------------------------
  switch( Dsu_type ) {
    case BIT24:
      Led_val6 = Digit_2;
      Led_val5 = Digit_4;
      Led_val4 = Digit_b;
      Led_val3 = Digit_i;
      Led_val2 = Digit_t;
      Led_val1 = Digit_NON;
      break;
    case BCD:
      Led_val6 = Digit_b;
      Led_val5 = Digit_c;
      Led_val4 = Digit_d;
      Led_val3 = Digit_NON;
      Led_val2 = Digit_NON;
      Led_val1 = Digit_NON;
      break;
    case BIT20:
      Led_val6 = Digit_2;
      Led_val5 = Digit_0;
      Led_val4 = Digit_b;
      Led_val3 = Digit_i;
      Led_val2 = Digit_t;
      Led_val1 = Digit_NON;
      break;
    case DISCON:
      Led_val6 = Digit_d;
      Led_val5 = Digit_i;
      Led_val4 = Digit_S;
      Led_val3 = Digit_c;
      Led_val2 = Digit_o;
      Led_val1 = Digit_n;
      break;
  }

  while( keyin_through(KEY_HALF) == SW_OFF ) {
    if( keyin_through(KEY_ZERO) == SW_ON ) {
      switch( Dsu_type ) {
        case BIT24:
          Dsu_type = BCD;
          Led_val6 = Digit_b;
          Led_val5 = Digit_c;
          Led_val4 = Digit_d;
          Led_val3 = Digit_NON;
          Led_val2 = Digit_NON;
          Led_val1 = Digit_NON;
          break;
        case BCD:
          Dsu_type = BIT20;
          Led_val6 = Digit_2;
          Led_val5 = Digit_0;
          Led_val4 = Digit_b;
          Led_val3 = Digit_i;
          Led_val2 = Digit_t;
          Led_val1 = Digit_NON;
          break;
        case BIT20:
          Dsu_type = DISCON;
          Led_val6 = Digit_d;
          Led_val5 = Digit_i;
          Led_val4 = Digit_S;
          Led_val3 = Digit_c;
          Led_val2 = Digit_o;
          Led_val1 = Digit_n;
          break;
        case DISCON:
          Dsu_type = BIT24;
          Led_val6 = Digit_2;
          Led_val5 = Digit_4;
          Led_val4 = Digit_b;
          Led_val3 = Digit_i;
          Led_val2 = Digit_t;
          Led_val1 = Digit_NON;
          break;
      }
      while( keyin_through(KEY_ZERO) == SW_ON ) { ; }
    }
  }

  switch( Dsu_type ) {
    case BIT24:  Write_b_eep( EEADR_DSU_TYPE, BIT24 ); break;
    case BCD:    Write_b_eep( EEADR_DSU_TYPE, BCD   ); break;
    case BIT20:  Write_b_eep( EEADR_DSU_TYPE, BIT20 ); break;
    case DISCON: Write_b_eep( EEADR_DSU_TYPE, DISCON); break;
  }
  delay_ms(20);
  disp_save();
  delay_ms(1000);

  Sys_mode = NORMAL;
  Timer2_enable = 1;  // START TMR2
}


//===============================================================================
// Enable DSU
//===============================================================================
void enable_dsu(byte dsu_type)
{
  //---- Enable Digital Scale Unit Interrupt -----------
  switch( dsu_type ) {
    case BIT24:
    case BCD:
      while( DSU_CLOCK == HIGH ) { ; }
      delay_ms(3);
      break;
    case BIT20:
      while(1) {
         while( DSU_CLOCK == HIGH )  { ; }
         if( DSU_CLOCK == LOW ) {
           Dsu_bit_pos = 0;
           break;   // detect end of data
         }
      }
      break;
  }


  Dummy = PORTB;
  dsu_colck_changed    = 0; // Clear INT1(PORTB1) interrupt flag
  dsu_colck_int_enable = 1; // Enable INT1 interrupts
  switch( dsu_type ) {
    case BIT24: dsu_colck_int_rising_edge = true;   break;
    case BCD:   dsu_colck_int_rising_edge = false;  break;  // test sitenai
    case BIT20:
           dsu_colck_int_rising_edge = false;
           dsu_data_changed = 0;     // Clear INT0(PORTB0) interrupt flag
           dsu_data_int_enable = 1;  // Enable INT0 interrupts
           dsu_data_int_rising_edge = false; 
           break;
  }
}


//===============================================================================
// Disable DSU
//===============================================================================
void disable_dsu()
{
  dsu_colck_int_enable = 0; // Enable PORTB4-7 interrupts
}


//===============================================================================
// set highspeed mode
//
// ArgNo  name            type    contents
// ------ -------------- ------- -----------------------------------
//  arg1  retry_count     byte   retry count
//===============================================================================
boolean set_highspeed( byte retry_count )
{
  byte    cnt;
  byte    clk;
  byte    prev_clk;
  boolean retry;

  retry = false;
  for( cnt = 1 ; cnt <= retry_count ; cnt++ ) {
    TMR1H = 0x00;
    TMR1L = 0x00;
    Timer1_overflowd = 0;
    prev_clk = HIGH;
    clk = 0;

    while( clk < 50 ) {
      if( DSU_CLOCK == LOW ) {
        Timer1_overflowd = 0;
        TMR1H = 0x00;
        TMR1L = 0x00;
        if( prev_clk == HIGH ) {
          clk++;
          prev_clk = LOW;
        }
      } else {
        prev_clk = HIGH;
      }
      if( Timer1_overflowd == 1 ) {
        DSU_MODE = LOW;
        delay_ms(200);
        DSU_MODE = HIGH;
        delay_ms(600);
        DSU_ZERO = LOW;
        delay_ms(200);
        DSU_ZERO = HIGH;
        break;
      }
    }
    delay_ms(500);
    retry = false;
    prev_clk = HIGH;
    clk = 0;

    TMR1H = 0x00;
    TMR1L = 0x00;
    Timer1_overflowd = 0;
    while( clk < 50 ) {
      if( DSU_CLOCK == LOW ) {
        Timer1_overflowd = 0;
        TMR1H = 0x00;
        TMR1L = 0x00;
        if( prev_clk == HIGH ) {
          clk++;
          prev_clk = LOW;
        }
      } else {
        prev_clk = HIGH;
      }
      if( Timer1_overflowd == 1 ) {
        retry = true;
        break;
      }
    }
    if( retry == false ) {
      return true;
    }
  }

  return false;
}


//===============================================================================
// disconect dsu
//===============================================================================
void discon( void )
{
  LED_DEV1 = LED_OFF;
  LED_DEV2 = LED_OFF;
  LED_DEV3 = LED_OFF;
  LED_DEV4 = LED_OFF;
  LED_DEV5 = LED_OFF;
  LED_DEV6 = LED_OFF;
  LED_SEG_A  = LED_OFF;
  LED_SEG_B  = LED_OFF;
  LED_SEG_C  = LED_OFF;
  LED_SEG_D  = LED_OFF;
  LED_SEG_E  = LED_OFF;
  LED_SEG_F  = LED_OFF;
  LED_SEG_G  = LED_OFF;
  LED_SEG_DP = LED_ON;
  while(1) {
    LED_DEV1 = LED_ON;
    delay_ms(1);
    LED_DEV1 = LED_OFF;
    delay_ms(5);
  }
}


//*******************************************************************************
// Digital Read Out  Product 6    Program main
//*******************************************************************************
void main()
{
  long dsu_tmp;
  long dsu_disp;
  long dsu_disp2;
  long dsu_div;
  long dsu_raw;
  long raw_save;       // 0.1mm move
  long tmp_longint;

  long mem1_ini;       // Origin coordinate
  long mem1_inisv;
  byte key2_stat;
  byte i;              // loop counter
  byte zero_key_stat;  // PUSHED / RELEASED
//  unsigned short init_counter;  // BCD highspeed mode


  //--- not in use --------------------------
  TRISCbits.RC5 = PORT_OUTPUT;
  TRISCbits.RC4 = PORT_OUTPUT;
  PORTCbits.RC5 = 0;
  PORTCbits.RC4 = 0;
  //-----------------------------------------
    
//  init_counter  = 0;    // dsu_type = BCD
  key2_stat     = false;
  zero_key_stat = RELEASED;

  dsu_disp  = 0;
  dsu_div   = 0;
  Dsu_dbl   = MAG_1X;

  dsu_raw   = 0;
  raw_save  = 0;

  Dsu_stat.DSU_SIGNAL_BIT = NOT_CHANGED;
  Dsu_stat.POWER_ON_BIT   = YES;
  Dsu_stat.POWER_ON_BIT2  = YES;
    
  Led_val1 = 0xFF; // all off
  Led_val2 = 0xFF;
  Led_val3 = 0xFF;
  Led_val4 = 0xFF;
  Led_val5 = 0xFF;
  Led_val6 = 0xFF;

  Led_pos   = 1;
  Sys_mode  = NORMAL;
  Tmr2_ovfc = 0;
  Tmr2_overflow_cnt = 0;
    
  //---- disable all interrupts -------------
  INTCONbits.GIEH = 0;          // 1:enable 0:disable interrupts
  INTCONbits.GIEL = 0;          // 1:enable 0:disable interrupts

  //---- 7segment LED port initialize (set OUTPUT) ----
  LED_DEV1_TRIS = PORT_OUTPUT;    // 7 SEGMENT LED 1
  LED_DEV2_TRIS = PORT_OUTPUT;    // 7 SEGMENT LED 2
  LED_DEV3_TRIS = PORT_OUTPUT;    // 7 SEGMENT LED 3
  LED_DEV4_TRIS = PORT_OUTPUT;    // 7 SEGMENT LED 4
  LED_DEV5_TRIS = PORT_OUTPUT;    // 7 SEGMENT LED 5
  LED_DEV6_TRIS = PORT_OUTPUT;    // 7 SEGMENT LED 6

  LED_SEG_A_TRIS  = PORT_OUTPUT;  // 7 SEGMENT LED SEGMENT A
  LED_SEG_B_TRIS  = PORT_OUTPUT;  // 7 SEGMENT LED SEGMENT B
  LED_SEG_C_TRIS  = PORT_OUTPUT;  // 7 SEGMENT LED SEGMENT C
  LED_SEG_D_TRIS  = PORT_OUTPUT;  // 7 SEGMENT LED SEGMENT D
  LED_SEG_E_TRIS  = PORT_OUTPUT;  // 7 SEGMENT LED SEGMENT E
  LED_SEG_F_TRIS  = PORT_OUTPUT;  // 7 SEGMENT LED SEGMENT F
  LED_SEG_G_TRIS  = PORT_OUTPUT;  // 7 SEGMENT LED SEGMENT G
  LED_SEG_DP_TRIS = PORT_OUTPUT;  // 7 SEGMENT LED SEGMENT DP

  LED_DEV1 =  LED_OFF;
  LED_DEV2 =  LED_OFF;
  LED_DEV3 =  LED_OFF;
  LED_DEV4 =  LED_OFF;
  LED_DEV5 =  LED_OFF;
  LED_DEV6 =  LED_OFF;

  //---- initialize EEPROM ------------------
  delay_ms(1000);

  if( Read_b_eep( EEADR_INIT ) == 0xFF ) {
    Write_b_eep( EEADR_INIT,     0x00   );
    Write_b_eep( EEADR_DIRECT,   MINUS  );
    Write_b_eep( EEADR_DOUBLE,   MAG_1X );
    Write_b_eep( EEADR_DSU_TYPE, BIT20  );
    delay_ms(20);
  }

  Dsu_type   = Read_b_eep( EEADR_DSU_TYPE );
  Dsu_direct = Read_b_eep( EEADR_DIRECT );
  Dsu_dbl    = Read_b_eep( EEADR_DOUBLE );

  //---- Port A Digital I/O -----------------
  CMCONbits.CM0 = 1;   // comparator off
  CMCONbits.CM1 = 1;
  CMCONbits.CM2 = 1;

  ADCON1bits.PCFG0 = 1;  // AN12-AN0 Digital I/O
  ADCON1bits.PCFG1 = 1;
  ADCON1bits.PCFG2 = 1;
  ADCON1bits.PCFG3 = 1;

  //---- disable port B pull-up -------------
  INTCON2bits.NOT_RBPU = 1;  // PortB pull-up disable

  //---- MODE & ZERO key initialize ---------
  DSU_MODE_TRIS = PORT_OUTPUT;
  DSU_ZERO_TRIS = PORT_OUTPUT;
  DSU_MODE = HIGH;
  DSU_ZERO = HIGH;

  //---- DSU port initialize (set INPUT) ----
  DSU_CLOCK_TRIS = PORT_INPUT;   // Digital Scale Unit CLOCK
  DSU_DATA_TRIS  = PORT_INPUT;   // Digital Scale Unit DATA

  //---- Internal switch initialize (set INPUT) -------
//  SW1_TRIS = PORT_INPUT;   MCLRE pin
  SW2_TRIS = PORT_INPUT;

  //---- TMR0 initialize --------------------
  Timer0_mode_8bit           = true;   // 8-Bit Mode
  Timer0_use_external_clock  = false;  // use internal instruction cycle
  Timer0_prescale_NOT_assign = false;  // Timer0 prescaler is assigned
  Timer0_prescale_bit2 = 1;            // Prescaler 1:32
  Timer0_prescale_bit1 = 0;            // Prescaler 1:32
  Timer0_prescale_bit0 = 0;            // Prescaler 1:32
  Timer0_overflowd = 0;                // Clear interrupt flag
  TMR0L = 0;
  Timer0_priority = Low_priority;
  Timer0_enable = 1;                   // TMR0 ON

  //---- TMR1 initialize --------------------
  Timer1_prescale_bit0 = 1;            // PreScaler 1:8
  Timer1_prescale_bit1 = 1;            // PreScaler 1:8
  Timer1_interrupt_enable    = false;  // Disable TMR1 interrupt
  Timer1_use_external_clock  = false;
  Timer1_priority            = Low_priority;
  Timer1_enable              = true;   // TMR1 ON

  //---- TMR2 initialize --------------------
  Timer2_postscale_bit3   = 1;           // PostScaler 1:16
  Timer2_postscale_bit2   = 1;           // PostScaler 1:16
  Timer2_postscale_bit1   = 1;           // PostScaler 1:16
  Timer2_postscale_bit0   = 1;           // PostScaler 1:16
  Timer2_prescale_bit1    = 1;           // PreScaler 1:16
  Timer2_prescale_bit0    = 1;           // PreScaler 1:16
  Timer2_priority         = Low_priority;
  Timer2_overflowd        = 0;           // Clear interrupt flag

  switch( Dsu_type ) { 
    case BIT24:
      if( set_highspeed( 4 ) == false ) {
        Led_val1 = Digit_E;
        Led_val2 = Digit_NON;
        Led_val3 = Digit_NON;
        Led_val4 = Digit_NON;
        Led_val5 = Digit_NON;
        Led_val6 = Digit_NON;
        delay_ms(500);
      }
      break;

    case BIT20:
      delay_ms(500);
      break;

    case BCD:
      // zero reset dsu
      DSU_ZERO = LOW;
      delay_ms(500);
      DSU_ZERO = HIGH;

      Led_val1 = 0x00;
      Led_val2 = 0x00;
      Led_val3 = 0x00;
      Led_val4 = 0x00;
      Led_val5 = 0x00;
      Led_val6 = 0x00;
      break;
  }
      
  //---- interrpt -------------------------------
  RCONbits.IPEN   = 1;              // 1:enable 0:disable priority levels
  INTCONbits.GIEH = 1;              // 1:enable 0:disable interrupts
  INTCONbits.GIEL = 1;              // 1:enable 0:disable interrupts
  Timer0_interrupt_enable = true;   // Enable interrupt
  Timer2_interrupt_enable = true;   // Enable interrupt    
  delay_ms(500);
  
  //---- Other setup MODE -------------------------------
  if( keyin_through(KEY_ZERO) == SW_ON ) {
    Char_pos = 1;
    other_setup();
    goto START_DRO;
  }

START_DRO:
  if( Dsu_type == DISCON ) {
    INTCONbits.GIEH = 0;              // 1:enable 0:disable interrupts
    INTCONbits.GIEL = 0;              // 1:enable 0:disable interrupts
    discon();
  }

  enable_dsu(Dsu_type);
  Sys_mode = NORMAL;

  Stop_count      = 0;
  Small_raw_count = 0;

  while( true ) {
    //---- display LED --------------------------
    if( Dsu_stat.DSU_SIGNAL_BIT == CHANGED ) {
      dsu_raw = get_dsu_rawdata();

      //------- smooth algorithm -----------------------
      Div_raw = Prev1_raw - dsu_raw;

      if( Prev2_raw == dsu_raw ) {
        if( Stop_count == 11 ) {
          if( Prev1_raw < Prev2_raw ) {
            Big_raw   = Prev2_raw;
            Small_raw = Prev1_raw;
          } else {
            Big_raw   = Prev1_raw;
            Small_raw = Prev2_raw;
          }
          if( Small_raw_count < Big_raw_count ) {
            dsu_raw = Big_raw;
          } else {
            dsu_raw = Small_raw;
          }
          Stop_count      = 0;
          Small_raw_count = 0;
          Big_raw_count   = 0;
        } else {
          if( Prev1_raw < Prev2_raw ) {
            Big_raw_count++;
          } else {
            Small_raw_count++;
          }
          Stop_count++;
          goto NO_DISP;
        }
      }
      if( Div_raw != 0 ) {
        Prev2_raw = Prev1_raw;
        Prev1_raw = dsu_raw;
      }
      //------- smooth algorithm end -------------------

      // output : dsu_raw
DISP:
      if( Dsu_stat.POWER_ON_BIT == YES ) {
        mem1_ini = dsu_raw;
        Dsu_stat.POWER_ON_BIT = NO;
      }

      dsu_tmp = dsu_raw - mem1_ini;
            
      switch( Dsu_type ) {
        case BIT24:
          dsu_div = dsu_tmp / 0x00000080L;
          dsu_disp = dsu_tmp - dsu_div;
          if( Dsu_direct == MINUS ) {
            dsu_disp2 = -dsu_disp;
          } else {
            dsu_disp2 = dsu_disp;
          }
          break;
        case BCD:
        case BIT20:
          if( Dsu_direct == MINUS ) {
            dsu_disp2 = -dsu_tmp;
          } else {
            dsu_disp2 = dsu_tmp;
          }
          break;
      }

      //------ move 0.1mm ----------------
      if( dsu_raw > raw_save ) {
        tmp_longint = dsu_raw - raw_save;
      } else {
        tmp_longint = raw_save - dsu_raw;
      }
      if( tmp_longint > 10 ) {  // 10 = 0.1mm
        key2_stat = false;
      }

      disp_led( dsu_disp2 );
      Dsu_stat.DSU_SIGNAL_BIT = NOT_CHANGED;
    }

NO_DISP:
    //---- input ZERO Key -----------------------------
    if( keyin_through(KEY_ZERO) == SW_ON ) {
      if( zero_key_stat == RELEASED ) {
        zero_key_stat = PUSHED;
        for( i = 1 ; i < 10 ; i++ ) {
          if( keyin_through(KEY_ZERO) == SW_OFF ) {
            break;
          }
          delay_ms(100);
        }
        if( i == 10 ) {
          mem1_ini = dsu_raw;
        }
      }
    }

    if( zero_key_stat == PUSHED ) {
      if( keyin_through(KEY_ZERO) == SW_OFF ) {
        zero_key_stat = RELEASED;
      }
    }

    //---- input 1/2 Key -----------------------------
    if( keyin_through(KEY_HALF) == SW_ON ) {
      if( key2_stat == false ) {
        key2_stat = true;
        mem1_inisv =  mem1_ini;
        raw_save   =  dsu_raw;
        dsu_tmp    =  dsu_raw - mem1_ini;
        dsu_tmp    /= 2;
        mem1_ini   += dsu_tmp;
      } else {
        key2_stat = false;
        mem1_ini = mem1_inisv;
      }
      while( keyin_through(KEY_HALF) == SW_ON ) { ; }
    }
        
    //---- dsu_type : BCD Highspeed mode ---------------------------
    /*
    if( Dsu_type == BCD ) {
      if( Dsu_stat.POWER_ON_BIT2 == YES ) {
        init_counter++;
        if( init_counter > 750 ) {  // approx 30sec
          INTCONbits.GIEH = 0;          // 1:enable 0:disable interrupts
          INTCONbits.GIEL = 0;          // 1:enable 0:disable interrupts
          LED_DEV1 = LED_OFF;
          LED_DEV2 = LED_OFF;
          LED_DEV3 = LED_OFF;
          LED_DEV4 = LED_OFF;
          LED_DEV5 = LED_OFF;
          LED_DEV6 = LED_OFF;
          DSU_MODE = LOW;
          delay_ms(100);
          DSU_MODE = HIGH;
          delay_ms(100);
          DSU_ZERO = LOW;
          delay_ms(100);
          DSU_ZERO = HIGH;
          Dsu_stat.POWER_ON_BIT2 = NO;
          INTCONbits.GIEH = 1;          // 1:enable 0:disable interrupts
          INTCONbits.GIEL = 1;          // 1:enable 0:disable interrupts
        }
      }
    }
    */
  }
}

//-------- E N D   O F   P R O G R A M ----------------------------------------
